(* ::Package:: *)

(* AutoDipole 1.2.1 - Package of an automated dipole subtraction by
Kouhei Hasegawa, Sven Moch, and Peter Uwer *)


(* virtualfortran.m *)


(* To convert I-terms into Fortran expression and to write down fortran files,
1.Iterm.f
2.checkVirt.f
3.nexternalVirt.inc *)


Virtualfortran:=Module[{},
targetVirtdir=StringJoin[targetdir,"/Virtual"];
CreateDirectory[targetVirtdir];
nameItermf=StringJoin[targetVirtdir,"/Iterm.f"];
namecheckVirtf=StringJoin[targetVirtdir,"/checkVirt.f"];
namenexternalVirtinc=StringJoin[targetVirtdir,"/nexternal.inc"];

str51=OpenWrite[nameItermf,FormatType->FortranForm,PageWidth->65];
createvirtualfortran;
Close[str51];
str52=OpenWrite[namecheckVirtf,FormatType->FortranForm,PageWidth->65];
writeItermcheckVirt;
Close[str52];
str53=OpenWrite[namenexternalVirtinc,FormatType->FortranForm,PageWidth->65];
writenexternalVirt;
Close[str53];
]


createvirtualfortran:=Module[{},                  
                  writeItermhead;
                  writeItermscalar;
                  writeItermcallCLB;
                  writeItermscoef;
                  writeItermSum;
                  endformatV[str51];
                  writeItermSubroutines;
]


writeItermhead:=Module[{},
WriteString[str51,
"c Generated by AutoDipole Version 1.2, \n",
"c Kouhei Hasegawa, Sven Moch, and Peter Uwer \n",
"c \n"];
WriteString[str51,"c Subroutine to calculate I terms \n"];
WriteString[str51,"c LO and Virtual process:",{{XV[1],p[1]},{XV[2],p[2]}}," --> ",tfinalV,"\n"];
WriteString[str51,"c I(i) = coef(i,-2)*eps^(-2) + coef(i,-1)*eps^(-1) + coef(i,0)  \n"];
WriteString[str51,"c SumI[-2,-1,0] = Sum_{i=1}^{", itotal, "}coef[-2,-1,0] \n\n"];
WriteString[str51,"      subroutine Iterm(p,coef,SumI) \n"];
WriteString[str51,"      implicit none \n","\n"];
WriteString[str51,"      integer i,j \n",
"      double precision p(0:3,1:",nfield,"),coef(",itotal,",-2:0),SumI(-2:0)\n",
"      double precision Pi,rtwo,Eul,AL,CF,CA,TR,mt,mb,dot,mu,Nf,log2\n"
];
WriteString[str51,"      double precision CLV(",itotal,"),q(0:3,1:",nfield,"),dilog\n"];
WriteString[str51,"      double precision "];
Do[ 
Do[
If[i != nfinalv+1 || j != nfinalv+2,WriteString[str51,"s",i,j,","]];
If[i == nfinalv+1 && j == nfinalv+2,WriteString[str51,"s",i,j]];
 ,{j,i+1,nfinalv+2}];
,{i,1,nfinalv+1}];
WriteString[str51," \n \n"];
WriteString[str51,"      common /MASS/ mt,mb  \n"];
WriteString[str51,"      common /usedalpha/ AL  \n"];
WriteString[str51,"    \n"];
WriteString[str51,
"      Pi=3.141592653589793238D0 \n",
"      Eul=0.5772156649015328606065120d0 \n",
"      rtwo=dsqrt(2.d0) \n",
"      log2=Log(2.d0) \n",
"      CF=4.D0/3.D0 \n",
"      CA=3.D0 \n",
"      TR=0.5D0 \n"];
WriteString[str51,"      mu=",NumberForm[mur,18],"D0 \n"];
WriteString[str51,"      Nf=",NumberForm[lightflavors,18],"D0 \n"];
WriteString[str51,"      \n"];
];


writeItermscalar:=Module[{},
Do[ 
Do[
    WriteString[str51,"      s",i,j,"=2.d0*dot(p(0,",i,"),p(0,",j,")) \n"];
,{j,i+1,nfinalv+2}];
,{i,1,nfinalv+1}];
WriteString[str51,"  \n"];
];


writeItermcallCLB:=Module[{},
WriteString[str51,
"      do i=0,3 \n",
"       q(i,1)=p(i,1) \n",
"       q(i,2)=p(i,2) \n",
"      enddo \n",
"      \n"];
Do[
If[ matchkind[i]==0, 
 WriteString[str51,
  "      call cmatrix1(p(0,1),",matchV[i],",CLV(",i,")) \n",
 (*"        CLV(",i,")=CLV(",i,")/",ng,".d0 \n",*)
  "      \n"];
];
If[ matchkind[i]==1, 
 WriteString[str51,
  "      call mexchange(q(0,1),p(0,1),",Part[exmomV[i],1],","
                      ,Part[exmomV[i],2] ,") \n",
  "      call cmatrix1(q(0,1),",matchV[i],",CLV(",i,")) \n",
  "      \n"];
];
If[ matchkind[i]==2,
 Do[
 WriteString[str51,
  "      call mmap(q(0,",k,"),p(0,",mapmom[i,k],"))\n"
  ];
,{k,3,nfield}];
 WriteString[str51,
  "      call cmatrix1(q(0,1),",matchV[i],",CLV(",i,"))\n",
  "      \n"];
];
,{i,1,itotal}];
];


writeItermscoef:=Module[{}, 
Do[Itermcoef[k,-2]=Itermcoef[k,-2]/.replog//Simplify,{k,1,itotal}]; 
Do[Itermcoef[k,-1]=Itermcoef[k,-1]/.replog//Simplify,{k,1,itotal}]; 
Do[Itermcoef[k,0]=Itermcoef[k,0]/.replog//Simplify,{k,1,itotal}]; 
Do[
  WriteString[str51,"      coef(",i,",-2)=\n"];
  WriteString[str51,"     - "];
  Write[str51,Itermcoef[i,-2]];
  WriteString[str51,"   \n"];

  WriteString[str51,"      coef(",i,",-1)=\n"];
  WriteString[str51,"     - "];
  Write[str51,Itermcoef[i,-1]];
  WriteString[str51,"   \n"];

  WriteString[str51,"      coef(",i,",0)=\n"];
  WriteString[str51,"     - "];
  Write[str51,Itermcoef[i,0]];
  WriteString[str51,"   \n"];

  ,{i,1,itotal}];
  WriteString[str51,"   \n"];
]


replog:={Log[2] -> log2, Log[64] -> 6*log2, Log[4096] -> 12*log2, Li2 -> dilog}


writeItermSum:=Module[{},
WriteString[str51,
"      SumI(-2) = 0.d0 \n",
"      SumI(-1) = 0.d0 \n",
"      SumI(-0) = 0.d0 \n",
" \n",
"      do i=1,",itotal," \n",
"      SumI(-2) = SumI(-2) + coef(i,-2) \n",
"      SumI(-1) = SumI(-1) + coef(i,-1) \n",
"      SumI(0) = SumI(0) + coef(i,0) \n",
"      enddo",
" \n \n"];
WriteString[str51," \n"];
]


writeItermSubroutines:=Module[{},
WriteString[str51,
"      subroutine mexchange(q,p,i,j)\n",
"      implicit none\n",
"      integer i,j,k,l\n",
"      double precision p(0:3,1:",nfield,"),q(0:3,1:",nfield,")\n",
"      do k=0,3\n",
"       do l=3,",nfield,"\n",
"        q(k,l) = p(k,l)\n",
"       enddo\n",
"       q(k,i) = p(k,j)\n",
"       q(k,j) = p(k,i)\n",
"      enddo\n",
"      return\n",
"      end\n",
" \n"];
WriteString[str51,
"      subroutine mmap(q,p)\n",
"      implicit none\n",
"      integer i\n",
"      double precision  p(0:3),q(0:3)\n",
"      do i=0,3\n",
"       q(i) = p(i)\n",
"      enddo\n",
"      return\n",
"      end\n",
" \n \n"];
]


writeItermcheckVirt:=Module[{},
WriteString[str52,
"c Generated by AutoDipole Version 1.2, \n",
"c Kouhei Hasegawa, Sven Moch, and Peter Uwer \n",
"c \n"];
WriteString[str52,
"      program checkVirt\n",
"      implicit none\n\n",
"      double precision p(0:3,1:",nfield,")\n",
"      double precision coef(",itotal,",-2:0),SumI(-2:0)\n",
"      integer i\n",
"      double precision msqLO,AL,pi,ALmsqLO,ratio,acc,mt,mb\n",
"      common /INPUTMV/ p\n",
"      common /usedalpha/ AL\n",
"      common /MASS/ mt,mb\n",
"      include 'coupl.inc'\n",
"      include 'nexternal.inc'\n",
"      call setpara('param_card.dat',.true.)\n",
"      pi=3.141592653589793238d0\n",
"      AL=g**2*(4.d0*pi)**(-1)\n",
"      mt=tmass\n",
"      mb=bmass\n",
"      call printframe\n",
"      do i=1,10\n",
"       call readmomenta2\n",
"       call smatrix(p,msqLO)\n",
"       ALmsqLO=msqLO*AL*(2.d0*pi)**(-1)\n",
"       call Iterm(p,coef,SumI)\n",
"       call accuracy(ALmsqLO,SumI(-2),ratio,acc)\n",
"       call printresult(i,SumI,ALmsqLO,ratio,acc)\n",
"\n",
"      enddo\n",
"      end\n",
"       \n",
"       \n",
"      subroutine readmomenta2\n",
"      implicit none\n",
"      integer idy1\n",
"      double precision dummy\n",
"      double precision p(0:3,1:",nfield,")\n",
"      common /INPUTMV/ p\n",
"       \n",
"      open (unit=11, file ='inputmLO.h', status = 'old')\n"
];
Do[WriteString[str52,
"      read  (11,*) idy1, p(0,",i,"), p(1,",i,"), p(2,",i,"), p(3,",i,")\n"
],{i,1,nfield}];
WriteString[str52,"      read  (11,*) dummy",
"  \n\n",
"      return\n",
"      end\n",
"       \n",
"      subroutine accuracy(standard,objective,ratio,acc)\n",
"      implicit none\n",
"      double precision standard,objective,ratio,acc\n",
"      ratio = objective/standard\n",
"      acc = (standard - objective)/standard\n",
"      return\n",
"      end\n",
"       \n",
"      double precision function dot(p1,p2) \n",
"      implicit none\n",
"      double precision p1(0:3),p2(0:3)\n",
"      dot=p1(0)*p2(0)-p1(1)*p2(1)-p1(2)*p2(2)-p1(3)*p2(3)\n",
"      end\n",
"       \n",
"\n",
"      subroutine printframe\n",
"      implicit none\n",
"      \n",
"      write(*,*) '     <I>                 =  c(-2)*eps^(-2) + c(-1)*eps\n",
"     &^(-1) + c(0)'\n",
"\n",
"      write(*,*) '|M(LO)|^2*Alpha/(2*pi)   =  b(0)'\n",
"      write(*,*) ' '\n",
"      write(*,*) '          c(-2)                  c(-1)\n",
"     &      c(0)'\n",
"      write(*,*) '          b(0)'\n",
"      write(*,*) '          c(0)/b(0)'\n",
"      write(*,*) ' '\n",
"      \n",
"      return\n",
"      end\n",
"      \n",
"      subroutine printresult(i,SumI,msqLO,ratio2,acc2)\n",
"      implicit none\n",
"      \n",
"      integer i\n",
"      double precision SumI(-2:0),msqLO,ratio2,acc2\n",
"      \n",
"      write(*,'(1x,i3)') i\n",
"      write(*,'(1x,4e27.15)') SumI(-2),SumI(-1),SumI(0)\n",
"      write(*,'(1x,1e27.15)') msqLO\n",
"      write(*,'(1x,1e27.15)') ratio2\n",
"      write(*,*) ' '\n",
"      \n",
"      return\n",
"      end\n",
"\n"]
]


endformatV[str_]:=Module[{},WriteString[str,"      return \n","      end \n","\n"];]


writenexternalVirt:=Module[{},
WriteString[str53,
"      integer    nexternal \n",
"      parameter (nexternal=  ",nfield,") \n",
"      integer    nincoming \n",
"      parameter (nincoming=  2) \n"];
]
